from pyperform import BenchmarkedFunction
from pyrsistent import pmap #!


class Benchmarked(BenchmarkedFunction):
    def __init__(self, scale=1, *args, **kwargs):
        super(Benchmarked, self).__init__(*args, timeit_number=scale*1000, **kwargs)

################# Create ###################

@Benchmarked()
def create_empty_pmap():
    for x in range(1000):
        _ = pmap()

@Benchmarked()
def reference_create_empty_dict():
    for x in range(1000):
        _ = dict()


def _small_dict():
    small_dict = dict((i, i) for i in range(10))


def _large_dict():
    large_dict = dict((i, i) for i in range(2000))


@Benchmarked(setup=_small_dict)
def create_small_pmap():
    for x in range(100):
        _ = pmap(small_dict)


@Benchmarked(setup=_small_dict)
def reference_create_small_dict():
    for x in range(100):
        _ = dict(small_dict)


@Benchmarked(setup=_large_dict)
def create_large_pmap():
    for x in range(1):
        _ = pmap(large_dict)


@Benchmarked(setup=_large_dict)
def reference_create_large_dict():
    for x in range(1):
        _ = dict(large_dict)


# ######################### Insert ######################


def _small_pmap():
    small_pmap = pmap(dict((i, i) for i in range(10)))


@Benchmarked(setup=_small_pmap)
def random_replace_small_pmap():
    for x in (9, 1, 4, 5, 7, 7, 3, 2):
        small_pmap.set(x, x)


@Benchmarked(setup=_small_dict)
def reference_random_replace_small_dict():
    for x in (9, 1, 4, 5, 7, 7, 3, 2):
        small_dict[x] = x


def _large_pmap():
    large_pmap = pmap(dict((i, i) for i in range(2000)))


@Benchmarked(setup=_large_pmap)
def random_replace_large_pmap():
    for x in (999, 111, 74, 1233, 6, 1997, 400, 1000):
        large_pmap.set(x, x)


@Benchmarked(setup=_large_dict)
def reference_random_replace_large_dict():
    for x in (999, 111, 74, 1233, 6, 1997, 400, 1000):
        large_dict[x] = x


@Benchmarked(setup=_small_pmap)
def random_replace_small_pmap_evolver():
    e = small_pmap.evolver()
    for x in (9, 1, 4, 5, 7, 7, 3, 2):
        e[x] = x
    m = e.persistent()


@Benchmarked(setup=_large_pmap)
def random_replace_large_pmap_evolver():
    e = large_pmap.evolver()
    for x in (999, 111, 74, 1233, 6, 1997, 400, 1000):
        e[x] = x
    m = e.persistent()


@Benchmarked(setup=_small_pmap)
def random_insert_new_small_pmap():
    for x in (19, 11, 14, 15, 17, 117, 13, 12):
        small_pmap.set(x, x)


@Benchmarked(setup=_small_dict)
def reference_random_insert_new_small_dict():
    for x in (19, 11, 14, 15, 17, 117, 13, 12):
        small_dict[x] = x


@Benchmarked(setup=_large_pmap)
def random_insert_new_large_pmap():
    for x in (100999, 100111, 10074, 1001233, 1006, 1001997, 100400, 1001000):
        large_pmap.set(x, x)


@Benchmarked(setup=_large_dict)
def reference_random_insert_new_large_dict():
    for x in (100999, 100111, 10074, 1001233, 1006, 1001997, 100400, 1001000):
        large_dict[x] = x


# ################## Read ########################

@Benchmarked(setup=_small_pmap)
def random_read_small_pmap():
    for x in (9, 1, 4, 5, 7, 7, 3, 2):
        y = small_pmap[x]


@Benchmarked(setup=_small_dict)
def reference_random_read_small_dict():
    for x in (9, 1, 4, 5, 7, 7, 3, 2):
        y = small_dict[x]


@Benchmarked(setup=_large_pmap)
def random_read_large_native_pvector():
    for x in (999, 111, 74, 1233, 6, 1997, 400, 1000):
        y = large_pmap[x]


@Benchmarked(setup=_large_dict)
def reference_random_read_large_list():
    for x in (999, 111, 74, 1233, 6, 1997, 400, 1000):
        y = large_dict[x]


# #################### Iteration #########################

@Benchmarked(setup=_large_pmap)
def iteration_large_pmap():
    for k in large_pmap:
        pass


@Benchmarked(setup=_large_dict)
def reference_iteration_large_dict():
    for k in large_dict:
        pass


# #################### Comparison ########################

def _different_pmaps_same_size():
    pmap1 = pmap(dict((i, i) for i in range(2000)))
    pmap2 = pmap(dict((i, i + 1) for i in range(2000)))


def _different_pmaps_different_size():
    pmap1 = pmap(dict((i, i) for i in range(2000)))
    pmap2 = pmap(dict((i, i + 1) for i in range(1500)))


def _equal_pmaps():
    pmap1 = pmap(dict((i, i) for i in range(2000)))
    pmap2 = pmap(dict((i, i) for i in range(2000)))


def _equal_pmap_and_dict():
    dict1 = dict((i, i) for i in range(2000))
    pmap1 = pmap(dict((i, i) for i in range(2000)))


def _equal_dicts():
    dict1 = dict((i, i) for i in range(2000))
    dict2 = dict((i, i) for i in range(2000))


def _different_dicts_same_size():
    dict1 = dict((i, i) for i in range(2000))
    dict2 = dict((i, i + 1) for i in range(2000))


def _different_dicts_different_size():
    dict1 = dict((i, i) for i in range(2000))
    dict2 = dict((i, i + 1) for i in range(2000))


def _equal_pmaps_different_bucket_size():
    pmap1 = pmap(dict((i, i) for i in range(2000)), 1999)
    pmap2 = pmap(dict((i, i) for i in range(2000)), 2000)


def _equal_pmaps_same_bucket_size_different_insertion_order():
    pmap1 = pmap([(i, i) for i in range(2000)], 1999)
    pmap2 = pmap([(i, i) for i in range(1999, -1, -1)], 1999)


@Benchmarked(setup=_large_pmap)
def compare_same_pmap():
    large_pmap == large_pmap


@Benchmarked(setup=_large_dict)
def reference_compare_same_dict():
    large_dict == large_dict


@Benchmarked(setup=_equal_pmaps)
def compare_equal_pmaps():
    pmap1 == pmap2


@Benchmarked(setup=_equal_dicts)
def reference_compare_equal_dicts():
    dict1 == dict2


@Benchmarked(setup=_equal_pmap_and_dict)
def compare_equal_pmap_and_dict():
    pmap1 == dict1


@Benchmarked(setup=_equal_pmaps_same_bucket_size_different_insertion_order)
def compare_equal_pmaps_different_insertion_order():
    pmap1 == pmap2


@Benchmarked(setup=_equal_pmaps_different_bucket_size)
def compare_equal_pmaps_different_bucket_size():
    pmap1 == pmap2


@Benchmarked(setup=_different_pmaps_same_size)
def compare_different_pmaps_same_size():
    pmap1 == pmap2


@Benchmarked(setup=_different_dicts_same_size)
def reference_compare_different_dicts_same_size():
    dict1 == dict2


@Benchmarked(setup=_different_pmaps_different_size)
def compare_different_pmaps_different_size():
    pmap1 == pmap2


@Benchmarked(setup=_different_dicts_different_size)
def reference_compare_different_dicts_different_size():
    dict1 == dict2



from pyperform import BenchmarkedFunction
from pyrsistent import _pvector, _pvector #!


class Benchmarked(BenchmarkedFunction):
    def __init__(self, scale=1, *args, **kwargs):
        super(Benchmarked, self).__init__(*args, timeit_number=scale*1000, **kwargs)

################# Create ###################

@Benchmarked()
def create_empty_native_pvector():
    for x in range(1000):
        _ = _pvector()

@Benchmarked()
def create_empty_python_pvector():
    for x in range(1000):
        _ = _pvector()

@Benchmarked()
def reference_create_empty_list():
    for x in range(1000):
        _ = list()

def _small_list():
    small_list = range(10)

def _large_list():
    large_list = range(2000)

@Benchmarked(setup=_small_list)
def create_small_native_pvector():
    for x in range(100):
        _ = _pvector(small_list)

@Benchmarked(setup=_small_list)
def create_small_python_pvector():
    for x in range(100):
        _ = _pvector(small_list)

@Benchmarked(setup=_small_list)
def reference_create_small_list():
    for x in range(100):
        _ = list(small_list)

@Benchmarked(setup=_large_list)
def create_large_native_pvector():
    for x in range(10):
        _ = _pvector(large_list)

@Benchmarked(setup=_large_list)
def create_large_python_pvector():
    for x in range(10):
        _ = _pvector(large_list)

@Benchmarked(setup=_large_list)
def reference_create_large_list():
    for x in range(10):
        _ = list(large_list)


####################### Append #####################

@Benchmarked()
def append_native_pvector():
    v = _pvector()
    for x in range(100):
        v = v.append(x)

@Benchmarked()
def append_python_pvector():
    v = _pvector()
    for x in range(100):
        v = v.append(x)

@Benchmarked()
def reference_append_list():
    l = []
    for x in range(100):
        l.append(x)

@Benchmarked()
def append_native_pvector():
    v = _pvector()
    for x in range(100):
        v = v.append(x)

@Benchmarked()
def append_python_pvector():
    v = _pvector()
    for x in range(100):
        v = v.append(x)

@Benchmarked()
def reference_append_list():
    l = []
    for x in range(100):
        l.append(x)


######################### Insert ######################

def _small_native_vector():
    small_native_vector = _pvector(range(10))


def _small_python_vector():
    small_python_vector = _pvector(range(10))


@Benchmarked(setup=_small_native_vector)
def random_insert_small_native_pvector():
    for x in (9, 1, 4, 5, 7, 7, 3, 2):
        small_native_vector.set(x, x)


@Benchmarked(setup=_small_python_vector)
def random_insert_small_python_pvector():
    for x in (9, 1, 4, 5, 7, 7, 3, 2):
        small_python_vector.set(x, x)


@Benchmarked(setup=_small_list)
def reference_random_insert_small_list():
    for x in (9, 1, 4, 5, 7, 7, 3, 2):
        small_list[x] = x


def _large_native_vector():
    large_native_vector = _pvector(range(2000))


def _large_python_vector():
    large_python_vector = _pvector(range(2000))


@Benchmarked(setup=_large_native_vector)
def random_insert_large_native_pvector():
    for x in (999, 111, 74, 1233, 6, 1997, 400, 1000):
        large_native_vector.set(x, x)


@Benchmarked(setup=_large_python_vector)
def random_insert_large_python_pvector():
    for x in (999, 111, 74, 1233, 6, 1997, 400, 1000):
        large_python_vector.set(x, x)


@Benchmarked(setup=_large_list)
def reference_random_insert_large_list():
    for x in (999, 111, 74, 1233, 6, 1997, 400, 1000):
        large_list[x] = x


@Benchmarked(setup=_small_native_vector)
def random_insert_small_native_pvector_evolver():
    e = small_native_vector.evolver()
    for x in (9, 1, 4, 5, 7, 7, 3, 2):
        e[x] = x
    v = e.persistent()


@Benchmarked(setup=_small_python_vector)
def random_insert_small_python_pvector_evolver():
    e = small_python_vector.evolver()
    for x in (9, 1, 4, 5, 7, 7, 3, 2):
        e[x] = x
    v = e.persistent()


@Benchmarked(setup=_large_native_vector)
def random_insert_large_native_pvector_evolver():
    e = large_native_vector.evolver()
    for x in (999, 111, 74, 1233, 6, 1997, 400, 1000):
        e[x] = x
    v = e.persistent()


@Benchmarked(setup=_large_python_vector)
def random_insert_large_native_pvector_evolver():
    e = large_python_vector.evolver()
    for x in (999, 111, 74, 1233, 6, 1997, 400, 1000):
        e[x] = x
    v = e.persistent()

################## Read ########################

@Benchmarked(setup=_small_native_vector)
def random_read_small_native_pvector():
    for x in (9, 1, 4, 5, 7, 7, 3, 2):
        y = small_native_vector[x]


@Benchmarked(setup=_small_python_vector)
def random_read_small_python_pvector():
    for x in (9, 1, 4, 5, 7, 7, 3, 2):
        y = small_python_vector[x]


@Benchmarked(setup=_small_list)
def reference_random_read_small_list():
    for x in (9, 1, 4, 5, 7, 7, 3, 2):
        y = small_list[x]


@Benchmarked(setup=_large_native_vector)
def random_read_large_native_pvector():
    for x in (999, 111, 74, 1233, 6, 1997, 400, 1000):
        y = large_native_vector[x]


@Benchmarked(setup=_large_python_vector)
def random_read_large_python_pvector():
    for x in (999, 111, 74, 1233, 6, 1997, 400, 1000):
        y = large_python_vector[x]


@Benchmarked(setup=_large_list)
def reference_random_read_large_list():
    for x in (999, 111, 74, 1233, 6, 1997, 400, 1000):
        y = large_list[x]


#################### Iteration #########################

@Benchmarked(setup=_large_native_vector)
def iteration_large_native_pvector():
    for x in large_native_vector:
        pass

@Benchmarked(setup=_large_python_vector)
def iteration_large_python_pvector():
    for x in large_python_vector:
        pass

@Benchmarked(setup=_large_list)
def reference_iteration_large_list():
    for x in large_list:
        pass


import os
from setuptools import setup, Extension
import sys
import platform
import warnings
import codecs
from distutils.command.build_ext import build_ext
from distutils.errors import CCompilerError
from distutils.errors import DistutilsPlatformError, DistutilsExecError
from _pyrsistent_version import __version__

readme_path = os.path.join(os.path.dirname(__file__), 'README.rst')
with codecs.open(readme_path, encoding='utf8') as f:
    readme = f.read()

extensions = []
if platform.python_implementation() == 'CPython':
    extensions = [Extension('pvectorc', sources=['pvectorcmodule.c'])]

needs_pytest = {'pytest', 'test', 'ptr'}.intersection(sys.argv)
pytest_runner = ['pytest-runner'] if needs_pytest else []


class custom_build_ext(build_ext):
    """Allow C extension building to fail."""

    warning_message = """
********************************************************************************
WARNING: Could not build the %s.
         Pyrsistent will still work but performance may be degraded.
         %s
********************************************************************************
"""

    def run(self):
        try:
            build_ext.run(self)
        except Exception:
            e = sys.exc_info()[1]
            sys.stderr.write('%s\n' % str(e))
            sys.stderr.write(self.warning_message % ("extension modules", "There was an issue with your platform configuration - see above."))

    def build_extension(self, ext):
        name = ext.name
        try:
            build_ext.build_extension(self, ext)
        except Exception:
            e = sys.exc_info()[1]
            sys.stderr.write('%s\n' % str(e))
            sys.stderr.write(self.warning_message % ("%s extension module" % name, "The output above this warning shows how the compilation failed."))

setup(
    name='pyrsistent',
    version=__version__,
    description='Persistent/Functional/Immutable data structures',
    long_description=readme,
    author='Tobias Gustafsson',
    author_email='tobias.l.gustafsson@gmail.com',
    url='http://github.com/tobgu/pyrsistent/',
    license='MIT',
    license_files=['LICENCE.mit'],
    py_modules=['_pyrsistent_version'],
    classifiers=[
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: Implementation :: PyPy',
    ],
    test_suite='tests',
    tests_require=['pytest<5', 'hypothesis<5'],
    scripts=[],
    setup_requires=pytest_runner,
    ext_modules=extensions,
    cmdclass={'build_ext': custom_build_ext},
    packages=['pyrsistent'],
    package_data={'pyrsistent': ['py.typed', '__init__.pyi', 'typing.pyi']},
    python_requires='>=3.5',
)
